-- Personal announcement.
function killing_personal_announcement(table, kills, killer, victim)
	--sv_msg2(killer, UT.COLOR[kills], UT.PERSONAL_ANNOUNCEMENT[kills]..'!@C')
	HUD_SHOW(killer, UT.COLOR[kills], UT.ANNOUNCEMENT.PERSONAL[kills], kills)
	ut_sound2(killer, table[kills])
end

function weapon_personal_announcement(kills, killer, weapon)
	--sv_msg2(killer, UT.COLOR[kills], player(killer, 'name')..' '..UT.PERSONAL_ANNOUNCEMENT[kills]..'!@C')
	HUD_SHOW(killer, WEAPON_AWARD_COLOR, string.upper(UT.ANNOUNCEMENT.WEAPON[weapon]), kills)
	ut_sound2(killer, UT.WEAPON[weapon][1])
end

-- Killing weapon general announcement.
function killing_weapon_announcement(kills, killer, weapon)
	sv_msg(BASE_COLOR, get_color(get_color_by_team(killer))..player(killer, 'name')..' '..get_color(WEAPON_AWARD_COLOR)..UT.ANNOUNCEMENT.WEAPON[weapon]..get_color(WEAPON_AWARD_COLOR)..'! ('..get_color('forestgreen')..itemtype(weapon, 'name')..get_color('forestgreen')..' mastery)')
	if GENERAL_SFX then
		ut_sound(UT.WEAPON[weapon][1], killer)
	end
end

-- Killing chain general announcement.
function killing_chain_announcement(kills, killer)
	sv_msg(BASE_COLOR, get_color(get_color_by_team(killer))..player(killer, 'name')..' '..UT.ANNOUNCEMENT.GENERAL[kills]..'!')
	if GENERAL_SFX then
		ut_sound(SFX.UT.CHAIN[kills], killer)
	end
end

-- Killing spree general announcement.
function killing_spree_announcement(kills, killer)
	sv_msg(BASE_COLOR, get_color(get_color_by_team(killer))..player(killer, 'name')..' '..UT.ANNOUNCEMENT.GENERAL[kills]..'! ('..kills..' kills)')
	if GENERAL_SFX then
		ut_sound(SFX.UT.SPREE[kills], killer)
	end
end

-- Returns the players name color for announcement by team.
function get_color_by_team(id)
	local team = player_team(id)
	local gamemode = tonumber(game("sv_gamemode"))
	if gamemode == DEATHMATCH then
		return DM_COLOR
	end
	if team == CT then
		return CT_COLOR
	elseif team == T then
		return T_COLOR
	end
end

-- Resets the player killing chain.
function reset_chain(params)
	local params = split(params, ",")
	local id = tonumber(params[1])
	local kills = tonumber(params[2])
	
	if PLAYER.CHAIN[id] == kills then
		PLAYER.CHAIN[id] = 0
	end
end

-- Verifies first round kill.
function on_first_kill(killer, victim)
	if FIRSTBLOOD then
		FIRSTBLOOD = false
		killing_personal_announcement(SFX.UT.START, 1, killer, victim)
		PLAYER.HUD.CANAL[killer] = 1
		if GENERAL_ANNOUNCEMENTS then
			sv_msg(BASE_COLOR, get_color(get_color_by_team(killer))..player(killer, 'name')..get_color(BASE_COLOR)..' '..UT.ANNOUNCEMENT.GENERAL[1]..'!')
			if GENERAL_SFX then
				ut_sound(SFX.UT.START[1], killer)
			end
		end
	end
end

-- Triggered each second (1000 ms, 60 FPS).
function on_second()
	for _, id in pairs(player(0, 'table')) do
		if PLAYER.CANAL[id] ~= 0 then
			if PLAYER.CANAL[id][2] == AWARD_KILLING_SPREE then
				killing_spree_announcement(PLAYER.CANAL[id][1], id)
			elseif PLAYER.CANAL[id][2] == AWARD_KILLING_CHAIN then
				killing_chain_announcement(PLAYER.CANAL[id][1], id)
			elseif PLAYER.CANAL[id][2] == AWARD_WEAPON_KILL then
				killing_weapon_announcement(PLAYER.CANAL[id][1], id, PLAYER.CANAL[id][3])
			end
			PLAYER.CANAL[id] = 0
		end
	end
end

-- Triggered on rounds begins (after spawn).
function on_startround(mode)
	FIRSTBLOOD = true
	ut_sound(SFX.UT.START[0])
end

-- Triggered on player join.
function on_join(id)
	HUD_INIT(id)
end

-- Triggered on player leave.
function on_leave(id)
	PLAYER.KILLS[id] = 0
	PLAYER.WEAPON[id] = INIT_TABLE(WEAPON_MAX_ID)
end

-- Triggered on player die.
function on_die(victim)
	PLAYER.KILLS[victim] = 0
end

-- Triggered on players hit.
function on_hit(victim, attacker, weapon, hpdmg, apdmg)
	-- Environment damage.
	if attacker == 0 or not is_firearm(weapon) then
		return
	end
	-- If friendly fire isn't enabled, team headshot isn't allowed.
	if tonumber(game("sv_friendlyfire")) == 0 then
		if player_team(victim) == player_team(attacker) then
			return
		end
	end
	-- Attacker rotation [0, 360) degrees.
	local attacker_angle = standard_angle(player(attacker, 'rot'));
	-- Angle between victim and attacker, with victim as origin.
	local angle = angle_from(player(victim, 'x'), player(victim, 'y'), player(attacker, 'x'), player(attacker, 'y'));
	-- Distance between victim and attacker.
	local distance = distance_between(player(victim, 'x'), player(victim, 'y'), player(attacker, 'x'), player(attacker, 'y'))
	-- Minimum precision for done a headshot, as distance function.
	local accuracy = calculate_accuracy(HEADSHOT_THRESHOLD, distance)
	-- How precise was the attack.
	local diff = math.abs(attacker_angle - angle)
	
	if diff < accuracy then
		on_headshot(victim, attacker, distance, weapon, hpdmg, apdmg)
	end
end

-- Triggered on headshot hit.
function on_headshot(victim, attacker, distance, weapon, hpdmg, apdmg)
	-- Victim armor.
	local victim_ap = player(victim, 'armor')
	-- Victim health.
	local victim_health = player(victim, 'health')
	-- Headshot damage.
	local shot_damage = HEADSHOT_BASE_DAMAGE * game('mp_damagefactor');
	-- If victim has helmet don't die.
	if victim_ap > 65 then
		setarmor(victim, victim_ap + apdmg)
		hit_armor(victim, 35)
		sv_sound3(player(victim, 'x'), player(victim, 'y'), SFX.PLAYER.HELMET, SFX.PLAYER.RADIUS)
	-- If victim has enough health can resist the headshot.
	elseif victim_health > shot_damage then
		sethealth(victim, victim_health + hpdmg)
		hit(victim, shot_damage)
		sv_sound3(player(victim, 'x'), player(victim, 'y'), SFX.PLAYER.HEADSHOT[math.random(1, 2)], SFX.PLAYER.RADIUS)
	-- Headshot kill.
	else
		headshot_kill(attacker, victim, weapon)
	end
end

-- Headshot kill.
function headshot_kill(killer, victim, weapon)
	local weapon_name = string.gsub(string.lower(itemtype(weapon, 'name')), '-', '')
	customkill(killer, victim, weapon_name, 'weapons/'..weapon_name..'_k.bmp')
	PLAYER.HUD.CANAL.HEADSHOT[killer] = PLAYER.KILLS[killer]
	HUD_HEADSHOT_SHOW(killer, 'Headshot!', PLAYER.KILLS[killer])
	sv_sound2(killer, SFX.UT.HEADSHOT)
	if GENERAL_ANNOUNCEMENTS then
		sv_msg(BASE_COLOR, get_color(get_color_by_team(killer))..player(killer, 'name')..get_color(BASE_COLOR)..' has done a '..get_color(HEADSHOT_COLOR)..'Headshot'..get_color(BASE_COLOR)..' kill!')
		if GENERAL_SFX then
			sv_sound(SFX.UT.HEADSHOT)
		end
	end
	sv_sound3(player(victim, 'x'), player(victim, 'y'), SFX.PLAYER.HEADSHOT[math.random(1, 2)], SFX.PLAYER.RADIUS)
end

-- Triggered on kill.
function on_kill(killer, victim, weapon)
	-- If weapon isn't recognized (cause a customkill) gets weapon id from killer.
	if weapon == 250 then
		weapon = player(killer, 'weapontype')
	end
	-- Environment kill.
	if killer == 0 or weapon > WEAPON_MAX_ID then
		return
	end

	-- Verifies first kill in round.
	on_first_kill(killer, victim)
	-- Player kills without die.
	PLAYER.KILLS[killer] = PLAYER.KILLS[killer] + 1
	-- Player killing chain.
	PLAYER.CHAIN[killer] = PLAYER.CHAIN[killer] + 1
	-- Player specific weapon kill counter.
	PLAYER.WEAPON[killer][weapon] = PLAYER.WEAPON[killer][weapon] + 1
	-- Timer for reset the chain counter.
	timer(KILLING_CHAIN_LAPSE, "reset_chain", killer..','..PLAYER.CHAIN[killer])

	-- Weapon mastery award.
	if INDEX_EXISTS(UT.WEAPON, weapon) then
		if PLAYER.WEAPON[killer][weapon] == UT.WEAPON[weapon][2] then
			weapon_personal_announcement(PLAYER.KILLS[killer], killer, weapon)
			PLAYER.HUD.CANAL[killer] = PLAYER.KILLS[killer]
			PLAYER.WEAPON[killer][weapon] = 0
			-- Sets players message canal with last player kill.
			if GENERAL_ANNOUNCEMENTS then
				PLAYER.CANAL[killer] = { PLAYER.KILLS[killer], AWARD_WEAPON_KILL, weapon }
			end
			return -- Priority over killing spree.
		end
	end
	
	-- Killing spree.
	if INDEX_EXISTS(SFX.UT.SPREE, PLAYER.KILLS[killer]) then
		killing_personal_announcement(SFX.UT.SPREE, PLAYER.KILLS[killer], killer, victim)
		PLAYER.HUD.CANAL[killer] = PLAYER.KILLS[killer]
		-- Sets players message canal with last player kill.
		if GENERAL_ANNOUNCEMENTS then
			PLAYER.CANAL[killer] = { PLAYER.KILLS[killer], AWARD_KILLING_SPREE }
		end
		return -- Priority over killing chain.
	end
	
	-- Killing chain.
	if INDEX_EXISTS(SFX.UT.CHAIN, PLAYER.CHAIN[killer]) then
		killing_personal_announcement(SFX.UT.CHAIN, PLAYER.CHAIN[killer], killer, victim)
		PLAYER.HUD.CANAL[killer] = PLAYER.CHAIN[killer]
		-- Sets players message canal with last player kill.
		if GENERAL_ANNOUNCEMENTS then
			PLAYER.CANAL[killer] = { PLAYER.CHAIN[killer], AWARD_KILLING_CHAIN }	
		end
	end
end