-- Converts the CS2D angle in standard angle,
-- as below:
--		 0
--	270		90
--		180
function standard_angle(angle) 
	if angle < 0 then 
		return angle + 360
	end
	return angle
end

-- Calculates the angle between 2 positions.
function angle_between(x1, y1, x2, y2)
	return math.deg(math.atan(math.abs(y2 - y1) / math.abs(x2 - x1)))
end

-- Calculates the distance between 2 positions.
function distance_between(x1, y1, x2, y2)
	return math.sqrt((x2 - x1) ^ 2 + ( y2 - y1 ) ^ 2)
end

-- Calculates the factor of specific
-- coordinates map quadrant of a object
-- relative to an origin.
-- It is intended for
-- transform the angle between two
-- positions into a angle of one
-- relative to another by its quadrants.
-- In CS2D quadrants and signs are as below:
-- 		Q3|Q4		(--)|(+-)
-- 		Q2|Q1		(+-)|(++)
-- x1 and y1 are origin coordinates.
-- x2 and y2 are object coordinates.
function quadrant_factor(x1, y1, x2, y2)
	local x = (x1 - x2)
	local y = (y1 - y2)
	if x > 0 then
		if y > 0 then
			return 90 -- Q1
		else
			return -90 -- Q4
		end
	else
		if y > 0 then
			return -270 -- Q2
		else
			return 270 -- Q3
		end
	end
end

-- Calculates the angle of one object
-- around an origin.
-- x1 and y1 are origin coordinates.
-- x2 and y2 are object coordinates.
-- Angle would be returned as below:
--		 0
--	270		90
--		180
-- Being the center the origin.
function angle_from(x1, y1, x2, y2)
	local angle = math.abs(angle_between(x1, y1, x2, y2) + quadrant_factor(x1, y1, x2, y2));
	if angle == 360 then
		return 0
	else
		return angle
	end
end

-- Reciprocal function for calculate accuracy as distance function.
function calculate_accuracy(threshold, distance)
	return threshold / distance
end