-- Initializes an array with defined size and default value.
function INIT_TABLE(size, value)
	local table = {}
	if value == nil then
		value = 0
	end
	for i = 1, size do
		table[i] = value
	end
	return table
end

-- Initializes an array with defined rows, columns and default value.
function INIT_MATRIX(rows, columns, value)
	local matrix = INIT_TABLE(rows, value)
	for i = 1, rows do
		matrix[i] = INIT_TABLE(columns, value)
	end
	return matrix
end

-- Verifies if a index exists in a table.
function INDEX_EXISTS(table, index)
	return table[index] ~= nil
end

-- Split a string by delimiter.
function split(string, delimiter)
    result = {};
    for match in (string..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match);
    end
    return result;
end

-- Initializes HUD for player messages.
function HUD_INIT(id)
	-- Calculates center of player screen.
	local x = player(id, 'screenw') / 2
	local y = player(id, 'screenh') / 2
	local vertical_percentage = -tonumber(y / 4);
	local headshot_diff = 24
	-- Calculates correct vertical position.
	if HUD.UT.VERTICAL_ALIGN == -1 then
		vertical_percentage = vertical_percentage * 2
	elseif HUD.UT.VERTICAL_ALIGN == 1 then
		vertical_percentage = vertical_percentage * -3
		headshot_diff = -24
	end
	-- Stores UT screen coordinates.
	PLAYER.HUD[id] = {
		x = x,
		y = y + vertical_percentage
	}
	-- Stores HEADSHOT screen coordinates.
	PLAYER.HUD.HEADSHOT[id] = {
		x = PLAYER.HUD[id].x,
		y = PLAYER.HUD[id].y + headshot_diff
	}
end

-- Shows the message to the player.
function HUD_SHOW(id, color, text, kills) 
	hudtxt2(id, HUD.UT.ID, color, text, PLAYER.HUD[id]['x'], PLAYER.HUD[id]['y'], 1, 1, HUD.UT.FONT_SIZE)
	timer(HUD.UT.DURATION, "HUD_HIDE", id..','..kills)
end

-- Shows the message to the player.
function HUD_HEADSHOT_SHOW(id, text, kills)
	hudtxt2(id, HUD.UT.HEADSHOT.ID, HEADSHOT_COLOR, text, PLAYER.HUD.HEADSHOT[id]['x'], PLAYER.HUD.HEADSHOT[id]['y'], 1, 1, HUD.UT.HEADSHOT.FONT_SIZE)
	timer(HUD.UT.DURATION, "HUD_HEADSHOT_HIDE", id..','..kills)
end

-- Hides the message.
-- First param is player ID.
-- Second parameter is message ID (kills).
function HUD_HEADSHOT_HIDE(params)
	local params = split(params, ",")
	local id = tonumber(params[1])
	local msg_id = tonumber(params[2])

	if PLAYER.HUD.CANAL.HEADSHOT[id] == msg_id then
		hudtxtalphafade(id, HUD.UT.HEADSHOT.ID, HUD.UT.FADE_OUT_TIME, 0)
	end
end

-- Hides the message.
-- Firs param is player ID.
-- Second parameter is message ID (kills).
function HUD_HIDE(params)
	local params = split(params, ",")
	local id = tonumber(params[1])
	local msg_id = tonumber(params[2])
	
	if PLAYER.HUD.CANAL[id] == msg_id then
		hudtxtalphafade(id, HUD.UT.ID, HUD.UT.FADE_OUT_TIME, 0)
	end
end

-- Loads general announcements.
function LOAD_ANNOUNCEMENTS()
	addhook("second", "on_second")
	-- Stores the latest chain or killing spree obtained of each player.
	-- First value is the award level and second the award types.
	PLAYER.CANAL = INIT_TABLE(32)
end

-- Unloads general announcements.
function UNLOAD_ANNOUNCEMENTS()
	freehook("second", "on_second")
	PLAYER.CANAL = nil
end